import cv2
from imutils.video import VideoStream
from imutils.video import FPS
import imutils
import argparse
import time
import numpy as np
from adafruit_servokit import ServoKit
from gpiozero import DistanceSensor
import serial

#Define distance sensor value
sensor = DistanceSensor(echo=24, trigger=18,max_distance = 4)

kit = ServoKit(channels = 16)

kit.servo[0].actuation_range = 180
kit.servo[1].actuation_range = 180
kit.servo[2].actuation_range = 180
kit.servo[3].actuation_range = 180

kit.servo[0].set_pulse_width_range(490,2250)
kit.servo[1].set_pulse_width_range(530,2400)
kit.servo[2].set_pulse_width_range(550,2500)
kit.servo[3].set_pulse_width_range(650,2600)

#Get Face recognition file
ap = argparse.ArgumentParser()
ap.add_argument("-f","--faces", required=True, help="path to the haar cascade faces file")

args = vars(ap.parse_args())

print("[INFO] loading model..")
faceCascade = cv2.CascadeClassifier(args["faces"])

print("[INFO] Opening Serial Port...")
ser = serial.Serial('/dev/ttyACM0', 9600, parity = serial.PARITY_NONE, stopbits = serial.STOPBITS_ONE, bytesize = serial.EIGHTBITS, timeout=1)
#time.sleep(2.0)
ser.flush()

# initialize the video stream, allow the camera sensor to warm up,
# and initialize the FPS counter
print("[INFO] starting video stream...")
vs = VideoStream(src=0).start()
time.sleep(2.0)
fps = FPS().start()

#initialize open cv reference frame
base = vs.read()
base = imutils.resize(base, height=320, width=480)

rows,cols,ch = base.shape
frameCenter = (int(cols/2),int(rows/2))
x_medium = int(cols/2)
y_medium = int(rows/2)

x_center = int(cols/2)
y_center = int(rows/2)
x_position = 90
y_position = 90
x_band = 10
y_band = 10

def initializeServos():
    kit.servo[0].angle =(90) 
    kit.servo[1].angle =(45)
    kit.servo[2].angle =(90) 
    kit.servo[3].angle =(90)
    time.sleep(.01)

initializeServos()


while True:
    # Capture frame-by-frame
    frame = vs.read()
    frame = imutils.resize(frame, height=320, width=480)

    gray = cv2.cvtColor(frame, cv2.COLOR_BGR2GRAY)


    faces = faceCascade.detectMultiScale(
        gray,
        scaleFactor=1.1,
        minNeighbors=5,
        minSize=(30, 30),
        flags=cv2.CASCADE_SCALE_IMAGE
    )

    # Draw a rectangle around the faces
    if len(faces)> 0:
        # extract the bounding box coordinates of the face and
        # use the coordinates to determine the center of the
        # face
        (x, y, w, h) = faces[0]
        cv2.rectangle(frame, (x, y), (x+w, y+h), (0, 255, 0), 2)
        
        faceX = int(x + (w / 2.0))
        faceY = int(y + (h / 2.0))

        # return the center (x, y)-coordinates of the face
        center = (faceX, faceY)
        cv2.circle(frame, center, 5, (255, 255, 0), -1)
        rectangle = faces[0]
    
    else:
        center = frameCenter
        faceX = center[0]
        faceY = center[1]
        cv2.circle(frame, center, 5, (255, 255, 0), -1)
        
        
    # Display the resulting frame
    cv2.imshow('Video', frame)
    
    # Move Horizontal servo motor
    if x_medium < faceX - x_band:
        x_position -= 1
    elif x_medium > faceX + x_band:
        x_position += 1
    # Move Vertiacl servo motor
    if y_medium < faceY- y_band:
        y_position += 1
    elif y_medium > faceY + y_band:
        y_position -= 1
    print("x =", x_position , "y =", y_position)          
    if x_position >= 180:
        x_position = 180
    elif x_position <+ 0:
        x_position = 0
    else:
        x_position = x_position
    if y_position >= 180:
        y_position = 180
    elif y_position <= 0:
        y_position = 0
    else:
        y_position = y_position
        
        
    #adjust the pan/tilt of camera servos    
    kit.servo[0].angle =(x_position) 
    kit.servo[1].angle =(y_position)
    

    #Include Sun tracker Values
    try: 
        val = ser.readline().decode('utf-8')
        parsed = val.split(',')
        parsed = [x.rstrip() for x in parsed]
        
    except UnicodeDecodeError or ValueError:
        ser.flush()
        val = ser.readline().decode('utf-8')
        parsed = val.split(',')
        parsed = [x.rstrip() for x in parsed]
    
    
    if (len(parsed)>1):
        #print (parsed)
        a = int(int(parsed[0]+'0')/10)
        b = int(int(parsed[1]+'0')/10)
        
        if a > 180:
            a = 180
        if b > 180:
            b = 180
        
        #get distance to face
        faceDistance = (sensor.distance)*100
        
        #find average mirror valuesq
        avgX = (a + x_position)/2 
        testX = (np.arctan((13/faceDistance))*180)/2
        mirrorX = 180 - int((a + testX)/2)
        
        avgY = ((b + y_position)/2) 
        testY = (np.arctan((13/faceDistance))*180)/2
        mirrorY = int(((b + testY)/2))

        
        if avgX > 180:
            avgX = 180
        elif avgX < 0:
            avgX = 0
        
        #Print out the angle values:
        print ("C1: " + str(x_position) + " C2: " + str(y_position))
        print ("M1: " + str(avgX) + " M2: " + str(avgY))
        print ("S1: " + str(a) + " S2: " + str(b))
        print ("Distance to Face: " + str(faceDistance))
        
        #Adjust mirror position based off of average values
        kit.servo[2].angle = avgX +5
        kit.servo[3].angle = avgY
   
    
    if cv2.waitKey(1) & 0xFF == ord('q'):
        break
    
    fps.update()


# stop the timer and display FPS information
fps.stop()
print("[INFO] elapsed time: {:.2f}".format(fps.elapsed()))
print("[INFO] approx. FPS: {:.2f}".format(fps.fps()))
# do a bit of cleanup
cv2.destroyAllWindows()
vs.stop()




